@@ -16,7 +16,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.08';
+$VERSION = '1.10';
 
 #------------------------------------------------------------------------------
 # Read or write information in a PPM/PGM/PBM image
@@ -48,19 +48,19 @@ sub ProcessPPM($$)
         # note: may contain comments starting with '#'
         if ($buff =~ /\G#/gc) {
             # must read more if we are in the middle of a comment
-            next unless $buff =~ /\G ?(.*\n(#.*\n)*)\s*/g;
+            next unless $buff =~ /\G ?(.*[\n\r]+(#.*[\n\r]+)*)\s*/g;
             $info{Comment} = $1;
             next if $buff =~ /\G#/gc;
         } else {
             delete $info{Comment};
         }
-        next unless $buff =~ /\G(\S+)\s+(\S+)\s/g;
+        next unless $buff =~ /\G(\S+)\s+(\S+)\s+/g;
         $info{ImageWidth} = $1;
         $info{ImageHeight} = $2;
         $type = [qw{PPM PBM PGM}]->[$num % 3];
         last if $type eq 'PBM'; # (no MaxVal for PBM images)
         if ($buff =~ /\G\s*#/gc) {
-            next unless $buff =~ /\G ?(.*\n(#.*\n)*)\s*/g;
+            next unless $buff =~ /\G ?(.*[\n\r]+(#.*[\n\r]+)*)\s*/g;
             $info{Comment} = '' unless exists $info{Comment};
             $info{Comment} .= $1;
             next if $buff =~ /\G#/gc;
@@ -76,7 +76,7 @@ sub ProcessPPM($$)
     }
     if (defined $info{Comment}) {
         $info{Comment} =~ s/^# ?//mg;   # remove "# " at the start of each line
-        $info{Comment} =~ s/\n$//;      # remove trailing newline
+        $info{Comment} =~ s/[\n\r]+$//; # remove trailing newline
     }
     $et->SetFileType($type);
     my $len = pos($buff);
@@ -114,7 +114,7 @@ sub ProcessPPM($$)
 #
     if ($verbose > 2) {
         print $out "$type header ($len bytes):\n";
-        HexDump(\$buff, $len, Out => $out);
+        $et->VerboseDump(\$buff, Len => $len);
     }
     my $tag;
     foreach $tag (qw{Comment ImageWidth ImageHeight MaxVal}) {
@@ -143,7 +143,7 @@ BitMap) images.
 
 =head1 AUTHOR
 
-Copyright 2003-2018, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2021, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
